package com.mortals.xhx.module.customer.service.impl;

import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.*;
import com.mortals.xhx.common.code.CustomerSatusEnum;
import com.mortals.xhx.module.customer.dao.CustomerDao;
import com.mortals.xhx.module.customer.dao.CustomerPwdRecordDao;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.model.CustomerEntityExt;
import com.mortals.xhx.module.customer.model.CustomerPwdRecordEntity;
import com.mortals.xhx.module.customer.model.CustomerQuery;
import com.mortals.xhx.module.customer.model.vo.CustomerCensusVo;
import com.mortals.xhx.module.customer.model.vo.CustomerDesignCensusVo;
import com.mortals.xhx.module.customer.service.CustomerService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
* CustomerService
* 客户管理 service实现
*
* @author zxfei
* @date 2022-06-08
*/
@Service("customerService")
public class CustomerServiceImpl extends AbstractCRUDServiceImpl<CustomerDao, CustomerEntity, Long> implements CustomerService {

    @Value("${sms.apiUrl:http://sms.wx3.com.cn/api/index/index}")
    private String smsApiUrl;
    @Value("${sms.appid:ADsUXLrS81vZDU95}")
    private String appid;

    /** 短信模板ID**/
    private static String SMS_TPYE = "30";

    private static String SMS_VERIFY_CODE_KEY ="login:sms:verify:";

    @Autowired
    private ICacheService cacheService;
    @Autowired
    private CustomerPwdRecordDao customerPwdRecordDao;

    @Override
    protected void saveBefore(CustomerEntity entity, Context context) throws AppException {
        this.validData(entity, context);
        if(StringUtils.isEmpty(entity.getContactTelphone())){
            throw new AppException("手机号不能为空");
        }
        if(StringUtils.isEmpty(entity.getCustName())){
            throw new AppException("姓名不能为空");
        }
        if(!PhoneUtil.isPhone(entity.getContactTelphone())){
            throw new AppException("手机号码格式不正确");
        }
        CustomerEntity query = new CustomerEntity();
        query.setContactTelphone(entity.getContactTelphone());
        List<CustomerEntity> list = dao.getList(query);
        if(CollectionUtils.isNotEmpty(list)){
            throw new AppException("手机号码为:"+entity.getContactTelphone()+"的用户已存在");
        }
        if (StringUtils.isNotEmpty(entity.getPassword())) {
            try {
                String saltKey = SecurityCode.getSecurityCode(6,SecurityCode.SecurityCodeLevel.Medium,false);
                String newPwd = entity.getPassword() + saltKey;
                newPwd = SecurityUtil.md5DoubleEncoding(newPwd);
                entity.setPassword(newPwd);
                entity.setSaltKey(saltKey);
            } catch (Exception e) {
                throw new AppException("密码转换异常");
            }
        } else {
            entity.setPassword(null);
        }
    }

    @Override
    protected CustomerEntity findBefore(CustomerEntity entity, PageInfo pageInfo, Context context) throws AppException {
        CustomerQuery query = new CustomerQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            CustomerQuery condition1 = new CustomerQuery();
            CustomerQuery condition2 = new CustomerQuery();
            CustomerQuery condition3 = new CustomerQuery();
            condition1.setContactTelphone(condition.toString());
            condition2.setLoginName(condition.toString());
            condition3.setCustName(condition.toString());
            List<CustomerQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            orConditionList.add(condition3);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }

    @Override
    public Result<CustomerEntityExt> findExt(CustomerEntity params, PageInfo pageInfo, Context context) throws AppException {
        CustomerEntity query = this.findBefore(params, pageInfo, context);
        Result<CustomerEntityExt> result = getDao().getListExt(query,pageInfo);
        List<CustomerEntityExt> list = result.getList();
        if(CollectionUtils.isNotEmpty(list)){
            list.stream().forEach(item->{
                item.setCreateTimeStr(DateUtils.getDateTime(item.getCreateTime(),DateUtils.P_yyyy_MM_dd_HH_mm_ss));
                item.setContactTelphone(DesensitizedUtil.mobilePhone(item.getContactTelphone()));
                item.setCustName(DesensitizedUtil.chineseName(item.getCustName()));
            });
        }
        return result;
    }

    @Override
    public CustomerEntity doLogin(String loginName, String password, String loginIp) throws AppException {
        CustomerEntity user = this.findByLoginName(loginName);
        if (user == null || !user.getLoginName().equals(loginName)) {
            throw new AppException("用户名不存在！");
        }
        if(user.getStatus()!= CustomerSatusEnum.NORMAL.getValue()){
            throw new AppException("该账号已被禁用！");
        }
        if(user.getErrorCount()==null){
            user.setErrorCount(0);
        }
        if(user.getLoginLock()==null){
            user.setLoginLock(0);
        }
        if(user.getLoginLock() == 1 && user.getLockTime()!=null){
            String nowStr = DateUtils.getCurrStrDate();
            String loginDateStr = DateUtils.getDateTime(user.getLockTime(),"yyyy-MM-dd");
            if(nowStr.equals(loginDateStr)) {
                throw new AppException("此账号当天密码错误次数已达上限，已被锁定");
            }
        }

        try {
            if(StringUtils.isNotEmpty(user.getSaltKey())) {
                password += user.getSaltKey();
            }
            if (!user.getPassword().equals(SecurityUtil.md5DoubleEncoding(password))) {
                CustomerEntity update = new CustomerEntity();
                update.setId(user.getId());
                update.setErrorCount(user.getErrorCount()+1);
                if(update.getErrorCount()==5){
                    update.setLoginLock(1);
                    update.setLockTime(new Date());
                }
                this.dao.update(update);
                throw new AppException("登录密码错误！");
            }else {
                if(user.getLastUpdatePwdTime()!=null) {
                    long day = DateUtils.getBetween(user.getLastUpdatePwdTime(), new Date(), 2);
                    if (day >= 90) {
                        throw new AppException("密码已超过三个月没有更改，请修改以后再登录");
                    }
                }
            }
        } catch (Exception e) {
            throw new AppException("密码验认出错！", e);
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(user.getId());
        update.setLastLoginAddress(loginIp);
        update.setLastLoginTime(new Date());
        this.update(update);
        return user;
    }

    private CustomerEntity findByLoginName(String loginName) {
        CustomerEntity params = new CustomerEntity();
        params.setLoginName(loginName);
        return this.selectOne(params);
    }

    @Override
    public CustomerEntity updateByPersonal(CustomerEntity params, Context context) throws AppException {
        CustomerEntity old = this.get(params.getId());
        if(old==null){
            throw new AppException("用户Id不正确！");
        }
        if(!old.getLoginName().equals(params.getLoginName())) {
            CustomerEntity user = this.findByLoginName(params.getLoginName());
            if (user != null) {
                throw new AppException("用户名已存在！");
            }
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(params.getId());
        update.setLoginName(params.getLoginName());
        update.setCustName(params.getCustName());
        update.setAvatar(params.getAvatar());
        update.setSex(params.getSex());
        update.setOrganization(params.getOrganization());
        update.setContactTelphone(params.getContactTelphone());
        update.setMailbox(params.getMailbox());
        update.setJob(params.getJob());
        update.setOpenId(params.getOpenId());
        int iRet = this.dao.update(update);
        if (iRet == 0) {
            throw new AppException(-1002, "更新失败！");
        } else {
            params.setPassword(null);
            return params;
        }
    }

    @Override
    public CustomerEntity changePassword(CustomerEntity params, Context context) throws AppException {
        if(StringUtils.isEmpty(params.getOldPassword())||StringUtils.isEmpty(params.getNewPassword())){
            throw new AppException("密码不能为空");
        }
        if(params.getOldPassword().equals(params.getNewPassword())){
            throw new AppException("新密码不能与旧密码一致");
        }
        CustomerEntity old = this.get(params.getId());
        if(old==null){
            throw new AppException("客户信息不存在");
        }
        String newPwd = "";
        String oldPwd = "";
        String saltKey = "";
        try {
            if(StringUtils.isNotEmpty(old.getSaltKey())) {
                oldPwd = params.getOldPassword()+old.getSaltKey();
            }else {
                oldPwd = params.getOldPassword();
            }
            oldPwd = SecurityUtil.md5DoubleEncoding(oldPwd);
            saltKey = SecurityCode.getSecurityCode(6,SecurityCode.SecurityCodeLevel.Medium,false);
            newPwd = params.getNewPassword() + saltKey;
            newPwd = SecurityUtil.md5DoubleEncoding(newPwd);
        }catch (Exception e) {
            throw new AppException("密码转换异常");
        }
        if(old.getPassword().equals(oldPwd)){
            CustomerEntity update = new CustomerEntity();
            update.setId(params.getId());
            update.setPassword(newPwd);
            update.setSaltKey(saltKey);
            this.update(update,context);
            old.setPassword(null);
            CustomerPwdRecordEntity userPwdRecordEntity = new CustomerPwdRecordEntity();
            userPwdRecordEntity.setUserId(old.getId());
            userPwdRecordEntity.setLoginName(old.getLoginName());
            userPwdRecordEntity.setLoginPwd(newPwd);
            userPwdRecordEntity.setCreateTime(new Date());
            userPwdRecordEntity.setCreateUserId(context.getUser().getId());
            customerPwdRecordDao.insert(userPwdRecordEntity);
        }else {
            throw new AppException("原来的密码不正确");
        }
        return old;
    }

    @Override
    public void changePasswordByAdmin(CustomerEntity params, Context context) throws AppException {
        CustomerEntity old = this.get(params.getId());
        if(old==null){
            throw new AppException("客户信息不存在");
        }
        String newPwd = "";
        String saltKey = "";
        try {
            saltKey = SecurityCode.getSecurityCode(6,SecurityCode.SecurityCodeLevel.Medium,false);
            newPwd = params.getNewPassword() + saltKey;
            newPwd = SecurityUtil.md5DoubleEncoding(newPwd);
        }catch (Exception e) {
            throw new AppException("密码转换异常");
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(params.getId());
        update.setPassword(newPwd);
        update.setSaltKey(saltKey);
        this.update(update,context);
        CustomerPwdRecordEntity userPwdRecordEntity = new CustomerPwdRecordEntity();
        userPwdRecordEntity.setUserId(old.getId());
        userPwdRecordEntity.setLoginName(old.getLoginName());
        userPwdRecordEntity.setLoginPwd(newPwd);
        userPwdRecordEntity.setCreateTime(new Date());
        userPwdRecordEntity.setCreateUserId(context.getUser().getId());
        customerPwdRecordDao.insert(userPwdRecordEntity);
    }

    @Override
    public Map<String, Object> getCustomerCensus() {
        Map<String, Object> result = new HashMap<>();
        List<CustomerCensusVo> customerCensusVos = dao.getCensus();
        CustomerDesignCensusVo designCensusVo = dao.getDesignCensus();
        if(CollectionUtils.isNotEmpty(customerCensusVos)){
            for(CustomerCensusVo item:customerCensusVos){
                if(item.getMemberLevel()==1){
                    result.put("trial",item.getCustomerCount());
                }
                if(item.getMemberLevel()==2){
                    result.put("vip",item.getCustomerCount());
                }
                if(item.getMemberLevel()==3){
                    result.put("designer",item.getCustomerCount());
                }
            }
        }
        result.put("picturesCount",designCensusVo.getPicturesCount());
        result.put("videosCount",designCensusVo.getVideosCount());
        return result;
    }

    @Override
    public void sendSmsVerifyCode(String mobileNumber) throws AppException {
        if(StringUtils.isEmpty(mobileNumber)){
            throw new AppException("手机号不能为空");
        }
        if(!PhoneUtil.isPhone(mobileNumber)){
            throw new AppException("手机号码格式不正确");
        }
        CustomerEntity customer = this.selectOne(new CustomerQuery().contactTelphone(mobileNumber));
        if(customer==null){
            throw new AppException("手机号码:" + mobileNumber + "没有注册用户");
        }
        String verifyCode = cacheService.get(SMS_VERIFY_CODE_KEY + mobileNumber);
        if(StringUtils.isNotEmpty(verifyCode)){
            throw new AppException("当前手机号码已发送验证码，请稍后重试");
        }
        try {
            Map<String, String> params = new HashMap<>();
            params.put("appid",appid);
            params.put("phone",mobileNumber);
            params.put("type",SMS_TPYE);

            String[] json = new String[2];
            String vCode = RandomUtil.randomNumbers(6);
            json[0] = vCode;
            json[1] = "1";
            params.put("json",JSONObject.toJSON(json).toString());
            String resp = HttpUtil.doPost(smsApiUrl,params);
            JSONObject respJson = JSONObject.parseObject(resp);
            if(respJson.getIntValue("code")==0){
                throw new AppException("短信发送失败:" + respJson.getString("message"));
            }
            //有效期60秒
            cacheService.setnx(SMS_VERIFY_CODE_KEY+mobileNumber,vCode,60);
        }catch (Exception e){
            log.error("短信发送异常",e);
            throw new AppException("短信发送异常");
        }

    }

    @Override
    public CustomerEntity doSmsLogin(String mobileNumber, String verifyCode, String loginIp) throws AppException {
        if(StringUtils.isEmpty(mobileNumber)){
            throw new AppException("手机号不能为空");
        }
        if(StringUtils.isEmpty(verifyCode)){
            throw new AppException("验证码不能为空");
        }
        if(!PhoneUtil.isPhone(mobileNumber)){
            throw new AppException("手机号码格式不正确");
        }
        CustomerEntity customer = this.selectOne(new CustomerQuery().contactTelphone(mobileNumber));
        if(customer==null){
            throw new AppException("手机号码:" + mobileNumber + "没有注册用户");
        }
        String vCode = cacheService.get(SMS_VERIFY_CODE_KEY + mobileNumber);
        if(StringUtils.isEmpty(vCode)){
            throw new AppException("验证码已失效");
        }
        if(!verifyCode.equals(vCode)){
            throw new AppException("验证码不正确");
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(customer.getId());
        update.setLastLoginAddress(loginIp);
        update.setLastLoginTime(new Date());
        this.update(update);
        return customer;
    }

    @Override
    public CustomerEntity doWeChatLogin(String openId, String loginIp) throws AppException {
        CustomerEntity customer = this.selectOne(new CustomerQuery().openId(openId));
        if(customer==null){
            throw new AppException("用户未绑定微信");
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(customer.getId());
        update.setLastLoginAddress(loginIp);
        update.setLastLoginTime(new Date());
        this.update(update);
        return customer;
    }

    public static void main(String[] args){
        try {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("appid","ADsUXLrS81vZDU95");
            jsonObject.put("phone","18981841597");
            jsonObject.put("type",SMS_TPYE);
            Map<String, String> params = new HashMap<>();
            params.put("appid","ADsUXLrS81vZDU95");
            params.put("phone","18981841597");
            params.put("type",SMS_TPYE);
            String vCode = RandomUtil.randomNumbers(6);

            params.put("json","[\"123456\",\"1\"]");
            List<String> jj = new ArrayList<>();
            jj.add("1111");
            jj.add("2");
            System.out.println(JSONObject.toJSON(jj).toString());
            String[] json = new String[2];
            json[0] = vCode;
            json[1] = "1";
            System.out.println(JSONObject.toJSON(json).toString());
            params.put("json",JSONObject.toJSON(jj).toString());
//            String resp = HttpUtil.doPost("http://sms.wx3.com.cn/api/index/index",params);
//            System.out.println(resp);
        }catch (Exception e){

        }
    }
}